%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%     Markovian Dynamics on Reaction Networks        %%%%%%%%%%%%
%%%%%%%%%%%           Example: Opinion formation               %%%%%%%%%%%%
%%%%%%%%%%%           RUN LINEAR NOISE APPROXIMATION           %%%%%%%%%%%%
%%%%%%%%%%%            Authors: Jenkinson/Goutsias             %%%%%%%%%%%%
%%%%%%%%%%%             Last Modified: 04/18/2012              %%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Compute RHS of covariance equations
%
function dCvec_dt = socialLYAP(t,Cvec,time_zeta,zeta,kt1,kt2,at1,at2,at3)


%
% Interpolate zeta(t)
%
zeta_t = zeros(4,1);

zeta_t(1)=interp1q(time_zeta,zeta(:,1),t);
zeta_t(2)=interp1q(time_zeta,zeta(:,2),t);
zeta_t(3)=interp1q(time_zeta,zeta(:,3),t);
zeta_t(4)=interp1q(time_zeta,zeta(:,4),t);

%
% Produce A(t) matrix
%
A = zeros(4,4);

A(1,1) = kt1*(1-(zeta_t(1)-zeta_t(2)))*exp(at1*(zeta_t(1)-zeta_t(2))...
               +at2*(zeta_t(3)-zeta_t(4))); 
A(2,2) = kt1*(1+(zeta_t(1)-zeta_t(2)))*exp(-at1*(zeta_t(1)-zeta_t(2))...
               -at2*(zeta_t(3)-zeta_t(4)));
A(3,3) = kt2*(1-(zeta_t(3)-zeta_t(4)))*exp(at3*(zeta_t(1)-zeta_t(2)));
A(4,4) = kt2*(1+(zeta_t(3)-zeta_t(4)))*exp(-at3*(zeta_t(1)-zeta_t(2)));

%
% Produce G(t) matrix
%
G = zeros(4,4);

G(1,1) = kt1*(at1-at1*zeta_t(1)+at1*zeta_t(2)-1)*...
              exp(at1*zeta_t(1)-at1*zeta_t(2)+at2*zeta_t(3)-at2*zeta_t(4));
G(2,1) = kt1*(-at1-at1*zeta_t(1)+at1*zeta_t(2)+1)*...
              exp(-at1*zeta_t(1)+at1*zeta_t(2)-at2*zeta_t(3)+at2*zeta_t(4));
G(3,1) = at3*kt2*(1-zeta_t(3)+zeta_t(4))*exp(at3*zeta_t(1)-at3*zeta_t(2));
G(4,1) = at3*kt2*(-1-zeta_t(3)+zeta_t(4))*exp(-at3*zeta_t(1)+at3*zeta_t(2));

G(1,2) = kt1*(-at1+at1*zeta_t(1)-at1*zeta_t(2)+1)*...
              exp(at1*zeta_t(1)-at1*zeta_t(2)+at2*zeta_t(3)-at2*zeta_t(4));
G(2,2) = kt1*(at1+at1*zeta_t(1)-at1*zeta_t(2)-1)*...
              exp(-at1*zeta_t(1)+at1*zeta_t(2)-at2*zeta_t(3)+at2*zeta_t(4));
G(3,2) = at3*kt2*(-1+zeta_t(3)-zeta_t(4))*exp(at3*zeta_t(1)-at3*zeta_t(2));
G(4,2) = at3*kt2*(1+zeta_t(3)-zeta_t(4))*exp(-at3*zeta_t(1)+at3*zeta_t(2));

G(1,3) = at2*kt1*(1-zeta_t(1)+zeta_t(2))*...
              exp(at1*zeta_t(1)-at1*zeta_t(2)+at2*zeta_t(3)-at2*zeta_t(4));
G(2,3) = at2*kt1*(-1-zeta_t(1)+zeta_t(2))*...
              exp(-at1*zeta_t(1)+at1*zeta_t(2)-at2*zeta_t(3)+at2*zeta_t(4));
G(3,3) = -kt2*exp(at3*zeta_t(1)-at3*zeta_t(2));
G(4,3) = kt2*exp(-at3*zeta_t(1)+at3*zeta_t(2));

G(1,4) = at2*kt1*(-1+zeta_t(1)-zeta_t(2))*...
              exp(at1*zeta_t(1)-at1*zeta_t(2)+at2*zeta_t(3)-at2*zeta_t(4));
G(2,4) = at2*kt1*(1+zeta_t(1)-zeta_t(2))*...
              exp(-at1*zeta_t(1)+at1*zeta_t(2)-at2*zeta_t(3)+at2*zeta_t(4));
G(3,4) = kt2*exp(at3*zeta_t(1)-at3*zeta_t(2));
G(4,4) = -kt2*exp(-at3*zeta_t(1)+at3*zeta_t(2));

%
% Generate Lyapunov equations
%

C=reshape(Cvec,4,4);

dC_dt = A + G*C + C*G';

dCvec_dt = reshape(dC_dt,16,1);