%exact Monte Carlo sampling vs. multiscale approximation
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%       Markovian Dynamics on Reaction Networks       %%%%%%%%%%%
%%%%%%%%%%%          Example: Transcription regulation          %%%%%%%%%%%
%%%%%%%%%%%            Authors: Jenkinson/Goutsias              %%%%%%%%%%%
%%%%%%%%%%%             Last Modified: 03/30/2012               %%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

clear all;
close all;

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% EXACT MONTE CARLO SAMPLING
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

tic;
display('Running exact Monte Carlo sampling. Please wait...');
exactMC;
toc

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% MULTISCALE APPROXIMATION
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

tic;
display('Running multiscale approximation method. Please wait...');
mscaleAPR;
toc

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% EXACT MEAN/STD COMPUTATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

iterExact = 2000;
totalTime = 35*60; % seconds
N         = 6;     % number of species

XdataExact = nan(iterExact,N,totalTime);

fid = fopen('results/exact.dat','r');
for i=1:iterExact
    XdataExact(i,:,:) = fread(fid,[N,totalTime],'double');
end
fclose(fid);

expectedmeanExact = squeeze(mean(XdataExact,1));
expectedstdExact  = squeeze(std(XdataExact,1));

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% APPROXIMATE MEAN/STD COMPUTATIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

iterApprox = 2000;

XdataAppr = nan(iterApprox,N,totalTime);

fid = fopen('results/approx.dat','r');
for i=1:iterApprox
    XdataAppr(i,:,:) = fread(fid,[N,totalTime],'double');
end
fclose(fid);

expectedmeanAppr = squeeze(mean(XdataAppr,1));
expectedstdAppr  = squeeze(std(XdataAppr,1));

%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%% MAKE PLOTS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%

lnwm   = 2;
lnws   = 1.5;
points = 0:(totalTime-1);
points = points/60;

sp=1;
figure, 
plot(points,expectedmeanExact(sp,:),'-','Color','b','LineWidth',lnwm);
hold on
plot(points,expectedmeanAppr(sp,:),'-','Color','r','LineWidth',lnwm);
hold on
legend('exact','approximate',1);
plot(points,expectedmeanAppr(sp,:)+expectedstdAppr(sp,:),'--','Color','r','LineWidth',lnws);
hold on
plot(points,expectedmeanAppr(sp,:)-expectedstdAppr(sp,:),'--','Color','r','LineWidth',lnws);
hold on
plot(points,expectedmeanExact(sp,:)+expectedstdExact(sp,:),'--','Color','b','LineWidth',lnws);
hold on
plot(points,expectedmeanExact(sp,:)-expectedstdExact(sp,:),'--','Color','b','LineWidth',lnws);
hold off
axis([0 35 0 15]);
xlabel('time (minutes)');
ylabel('population');
title('X_1')
saveas(gcf,'results/x1.fig')

sp=2;
figure, 
plot(points,expectedmeanExact(sp,:),'-','Color','b','LineWidth',lnwm);
hold on
plot(points,expectedmeanAppr(sp,:),'-','Color','r','LineWidth',lnwm);
hold on
legend('exact','approximate',2);
plot(points,expectedmeanAppr(sp,:)+expectedstdAppr(sp,:),'--','Color','r','LineWidth',lnws);
hold on
plot(points,expectedmeanAppr(sp,:)-expectedstdAppr(sp,:),'--','Color','r','LineWidth',lnws);
hold on
plot(points,expectedmeanExact(sp,:)+expectedstdExact(sp,:),'--','Color','b','LineWidth',lnws);
hold on
plot(points,expectedmeanExact(sp,:)-expectedstdExact(sp,:),'--','Color','b','LineWidth',lnws);
hold off
axis([0 35 0 50]);
xlabel('time (minutes)');
ylabel('population');
title('X_2')
saveas(gcf,'results/x2.fig')

sp=3;
figure, 
plot(points,expectedmeanExact(sp,:),'-','Color','b','LineWidth',lnwm);
hold on
plot(points,expectedmeanAppr(sp,:),'-','Color','r','LineWidth',lnwm);
hold on
legend('exact','approximate',2);
plot(points,expectedmeanAppr(sp,:)+expectedstdAppr(sp,:),'--','Color','r','LineWidth',lnws);
hold on
plot(points,expectedmeanAppr(sp,:)-expectedstdAppr(sp,:),'--','Color','r','LineWidth',lnws);
hold on
plot(points,expectedmeanExact(sp,:)+expectedstdExact(sp,:),'--','Color','b','LineWidth',lnws);
hold on
plot(points,expectedmeanExact(sp,:)-expectedstdExact(sp,:),'--','Color','b','LineWidth',lnws);
hold off
axis([0 35 0 170]);
xlabel('time (minutes)');
ylabel('population');
title('X_3')
saveas(gcf,'results/x3.fig')

sp=4;
figure, 
plot(points,expectedmeanExact(sp,:),'-','Color','b','LineWidth',lnwm);
hold on
plot(points,expectedmeanAppr(sp,:),'-','Color','r','LineWidth',lnwm);
hold on
legend('exact','approximate',1);
plot(points,expectedmeanAppr(sp,:)+expectedstdAppr(sp,:),'--','Color','r','LineWidth',lnws);
hold on
plot(points,expectedmeanAppr(sp,:)-expectedstdAppr(sp,:),'--','Color','r','LineWidth',lnws);
hold on
plot(points,expectedmeanExact(sp,:)+expectedstdExact(sp,:),'--','Color','b','LineWidth',lnws);
hold on
plot(points,expectedmeanExact(sp,:)-expectedstdExact(sp,:),'--','Color','b','LineWidth',lnws);
hold off
axis([0 35 0 2.5]);
xlabel('time (minutes)');
ylabel('population');
title('X_4')
saveas(gcf,'results/x4.fig')

sp=5;
figure, 
plot(points,expectedmeanExact(sp,:),'-','Color','b','LineWidth',lnwm);
hold on
plot(points,expectedmeanAppr(sp,:),'-','Color','r','LineWidth',lnwm);
hold on
legend('exact','approximate',1);
plot(points,expectedmeanAppr(sp,:)+expectedstdAppr(sp,:),'--','Color','r','LineWidth',lnws);
hold on
plot(points,expectedmeanAppr(sp,:)-expectedstdAppr(sp,:),'--','Color','r','LineWidth',lnws);
hold on
plot(points,expectedmeanExact(sp,:)+expectedstdExact(sp,:),'--','Color','b','LineWidth',lnws);
hold on
plot(points,expectedmeanExact(sp,:)-expectedstdExact(sp,:),'--','Color','b','LineWidth',lnws);
hold off
axis([0 35 0 1.5]);
xlabel('time (minutes)');
ylabel('population');
title('X_5')
saveas(gcf,'results/x5.fig')

sp=6;
figure, 
plot(points,expectedmeanExact(sp,:),'-','Color','b','LineWidth',lnwm);
hold on
plot(points,expectedmeanAppr(sp,:),'-','Color','r','LineWidth',lnwm);
hold on
legend('exact','approximate',2);
plot(points,expectedmeanAppr(sp,:)+expectedstdAppr(sp,:),'--','Color','r','LineWidth',lnws);
hold on
plot(points,expectedmeanAppr(sp,:)-expectedstdAppr(sp,:),'--','Color','r','LineWidth',lnws);
hold on
plot(points,expectedmeanExact(sp,:)+expectedstdExact(sp,:),'--','Color','b','LineWidth',lnws);
hold on
plot(points,expectedmeanExact(sp,:)-expectedstdExact(sp,:),'--','Color','b','LineWidth',lnws);
hold off
axis([0 35 0 2.5]);
xlabel('time (minutes)');
ylabel('population');
title('X_6')
saveas(gcf,'results/x6.fig')